@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 
 import pandas
-from utils import dmesg
 
 from base_parser_class import InteropBinParser
+from utils import dmesg, set_column_sequence
 
 class InteropQualityMetrics(InteropBinParser):
     "ILMN Quality metrics parser (child class of InteropBinParser)."
@@ -18,19 +18,35 @@ class InteropQualityMetrics(InteropBinParser):
         dmesg("QM Read Config:", 2)
         dmesg(self.read_config, 2)
         self._setup_read_tiers()
+        
+        # create .qcol_sequence to retain correct order of qx columns
+        self._set_qcol_sequence()
+        
+        # create data dict for parser, making use of .qcol_sequence
         self._setup_data()
 
-        # filled at the end of parsing.
+        # Q30 / Q20 scores per read, calculated at end of parsing.
         self.read_qscore_results = {'readnum': [], 'q30': [], 'q20': [] }
 
+    def _set_qcol_sequence(self):
+        "sets .qcol_sequence array of q score labels in correct order."
+        self.qcol_sequence = []
+        for qual in range(1,self.num_quality_scores+1):
+            self.qcol_sequence.append('q'+str(qual))
+
     def _setup_data(self):
         self.data = { 'lane': [], 'cycle': [], 'tile': [] }
-        for qual in range(1,self.num_quality_scores+1):
-            self.data['q'+str(qual)] = []
+        
+        for qual in self.qcol_sequence:
+            self.data[qual] = []
 
     def _setup_read_tiers(self):    
-        """concocts a 'read tier' array describing the start-number of cycles per individual Read.
-        (Example: read_config of 151/6/151 becomes read_tiers=[151,157,308]""" 
+        """
+        Concocts a 'read tier' array describing the start-number of cycles per 
+        individual Read.
+        
+        (E.g. read_config of 151/6/151 becomes read_tiers=[151,157,308]
+        """ 
 
         self.read_tiers = []     # list of tuples
 
@@ -43,18 +59,32 @@ class InteropQualityMetrics(InteropBinParser):
             last_tier = new_tier
 
         dmesg(self.read_tiers, 3)
+        
+    def get_df_col_sequence(self):
+        "returns array of column names in correct order for DataFrame (.df)"
+        out = ['cycle', 'lane', 'tile']
+        out.extend(self.qcol_sequence)
+        return out
 
     def index_quality(self, target_qscore=30):
         "Convenience method to return index read's % quality for target_qscore (default 30)"
     
-        # Assumes only one Index read, which is true for MiSeqs but not true for HiSeqs. 
-        # WillNotFix
+        # Assumes only one Index read, which is not a good assumption. 
+        # No plans to fix unless specifically requested. (Would probably make more
+        # sense just to get rid of this function. -nm) 
+        
         for read in self.read_config:
             if read['is_index']:
                 return self.get_qscore_percentage(target_qscore, read['read_num']-1)
 
     def get_qscore_percentage(self, target_qscore=30, read_num=-1):
-        "Returns % of quality scores at or above target_qscore (default 30). 2nd argument specifies read number."
+        """Returns PERCENTAGE of quality scores at or above target_qscore. 
+        
+        Supplying read_num=-1 returns qscore percentage across all reads. 
+        
+        :param target_qscore: int designates target quality level (default: 30)
+        :param read_num: int specifies read number (default: -1)."
+        """
     
         q_upper_cols = [x for x in self.data.keys() if x[0]=='q' and int(x[1:]) > target_qscore-1 ]
         q_upper_df = self.idf[q_upper_cols] 
@@ -65,7 +95,7 @@ class InteropQualityMetrics(InteropBinParser):
             q_total_sum = self.idf.values.sum()
 
         else:
-            # segment Qscores by read_num. Let IndexError be raised in case of too-high read_num.
+            # segment Qscores by read_num. Let IndexError be raised for too-high read_num.
             # read_tiers example: [151,157,308] 
         
             cycle_start = 0 if read_num==0 else self.read_tiers[read_num-1] + 1 
@@ -82,10 +112,14 @@ class InteropQualityMetrics(InteropBinParser):
             q_total_sum = self.idf[i_start:i_end].values.sum()  
         
         # Return a percentage (like in Illumina SAV)
-        return 100 * float(q_upper_sum) / float(q_total_sum)
+        if q_total_sum:
+            return 100 * float(q_upper_sum) / float(q_total_sum)
+        else:
+            return 0
 
     def parse_binary(self):
-        "Important: set read_config appropriately -- needed to construct read_tiers to separate Q scores by Read."
+        """ Do the work.  Important: set read_config appropriately, which is
+            needed to construct read_tiers to separate Q scores by Read."""
 
         bs = self.bs   
     
@@ -117,7 +151,7 @@ class InteropQualityMetrics(InteropBinParser):
             for qual in range(1, self.num_quality_scores + 1):  #(50 entries of 4 bytes each)
                 self.data['q'+str(qual)].append(bs.read('uintle:32'))
 
-        self.df = pandas.DataFrame(self.data)
+        self.df = set_column_sequence(pandas.DataFrame(self.data), self.qcol_sequence)
     
         self.idf = self.make_coordinate_plane(self.df, flatten=True)
         
@@ -135,9 +169,11 @@ class InteropQualityMetrics(InteropBinParser):
                                     "(Index)" if read['is_index'] else "")
         return out
 
-    #def to_dict(self):
-    #TODO: Make a dict that reports on Q30 and Q20 scores of each read.
-    #    return self.read_qscore_results
+    def to_dict(self, target_qscore=30):
+        out = {}
+        for read in self.read_config:
+            out[read['read_num']] = self.get_qscore_percentage(target_qscore, read['read_num']-1)
+        return out
 
 
 if __name__=='__main__':
@@ -154,6 +190,6 @@ if __name__=='__main__':
     
     print 'Length of data: %i' % len(QM.data['cycle'])
 
-    qscore_df = pandas.DataFrame(QM.to_dict())
-    print qscore_df
-    
+    print "Quality Score Histogram (all reads)"
+    print QM.idf.sum()
+    
\ No newline at end of file
